{# Render a read.render.renderable_calculator.TextItem #}
<span id="{{ item.span_id }}"
      class="{{ item.html_class_string }}"
      data-lang-id="{{ item.lang_id }}"
      data-paragraph-id="{{ item.para_id }}"
      data-sentence-id="{{ item.se_id }}"
      data-text="{{ item.text }}"
      data-status-class="{{ item.status_class }}"
      data-order="{{ item.order }}"
{% if item.wo_id is not none %}
      data-wid="{{ item.wo_id }}"
{% endif %}>{{ item.html_display_text | safe }}</span>
