body {
  --status-0-color: #59c7fa;
  --status-1-color: #7a3d58;
  --status-2-color: #82621a;
  --status-3-color: #3c6621;
  --status-4-color: #325192;
  --status-5-color: #5e4583;

  --status-99-color: #38924c;

  --background-color: #121212; /* black paper */
  --font-color: #F3F4F4;
  --light-grey: #c4c8ce;

  background-color: var(--background-color);
  color: var(--font-color); /* grey font */
}

.flash-notice {
  background-color: #7a3d58; /* pink highlight */
}

.menu-item > span {
    color: var(--light-grey) !important; /* light grey font */
}

a.home-link {
    color: var(--light-grey) !important; /* light grey font */
}

a {
  color: var(--status-0-color); /* ice blue */
}

a:link {
  color: var(--status-0-color); /* ice blue */
}

a:visited {
  color: var(--status-0-color); /* ice blue */
}

a:hover {
  color: #FF5C5C; /* red */
}

input[type="text"],
textarea,
.ui-widget-content,
.sub-menu {
    border-color: var(--light-grey); /* light grey */
    background-color: var(--background-color); /* black */
    color: var(--light-grey); /* light grey font */
}

.tagify {
  --tags-border-color: var(--light-grey);
  background-color: var(--background-color);
  color: var(--light-grey);
  --placeholder-color-focus: var(--light-grey);
}

.tagify--focus {
  --tags-border-color: var(--tagify__tag-bg);
}

div#thetext p { line-height: 1.5; }
span.textitem { font-family: Georgia;}
span.status0 { background-color: transparent; color: var(--status-0-color); } /* ice blue */
span.status1 { background-color: var(--status-1-color); } /* pink highlight */
span.status2 { background-color: var(--status-2-color); } /* yellow highlight */
span.status3 { background-color: var(--status-3-color); } /* green highlight */
span.status4 { background-color: var(--status-4-color); } /* blue highlight */
span.status5 { background-color: var(--status-5-color); } /* purple highlight */
span.status98 { background-color: transparent; } /* transparent */
span.status99 { background-color: transparent; } /* transparent */

#status {
  --status-select-color: #d7dce2;
  --status-hover-color: var(--light-grey);
}

#status label[for="status-5"]::after,
#status label[for="status-6"]::after,
#status #status-5:checked + label::after,
#status #status-6:checked + label::after {
    filter: invert(1);
}

#status label[for="status-6"]:hover {
  background-color: #bc6b60;
}

.status-bar-container {
  background-color: #fff;
  border-color: var(--light-grey);
}

.status-bar-container-empty {
  background-color: var(--background-color);
  border-color: #686868;
}

.status-bar-container span {
  color: var(--background-color);
}

#read-slider {
  -webkit-appearence: none;
  background-color: #5e5e5e;
}

span.hamburger {
  background-color: #F3F4F4;
}

#reading_menu .reading-menu-item:hover {
  color: var(--background-color);
}